/*
 * Creado el da 27/05/2005
 * Ejemplo de Conexin a CDS Server desde una aplicacin Java
 */
package com.dizares.cds.client;

import java.rmi.RemoteException;
import java.security.acl.AclNotFoundException;
import java.util.Properties;
import java.util.Vector;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.naming.CommunicationException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.swing.JOptionPane;

import com.dizares.cds.persistence.EJBPersistence;
import com.dizares.cds.persistence.interfaces.EBCConnector;
import com.dizares.cds.persistence.interfaces.EBCConnectorHome;
import com.dizares.cds.value.Course;
import com.dizares.node.PersistenceManager;
import com.dizares.node.PersistenceSelector;

public class Sample {

	public static void main(String[] args) {
		
		// Ac configuro las propiedades para la conexin con JBoss
		Properties sysProp = System.getProperties();
		
		// Le indico la clase que ser utilizada para crear la conexin con el
		// servicio de nombres.
		sysProp.setProperty("java.naming.factory.initial",
				"org.jnp.interfaces.NamingContextFactory");
		
		// esto no se para qu... :-)
		sysProp.setProperty("java.naming.factory.url.pkgs",
				"org.jboss.naming:org.jnp.interfaces");
		
		// ac le decs la IP dnde se encuentra el JBoss
		sysProp.setProperty("java.naming.provider.url", "localhost");
		
		// en caso de que pases a travs de un PROXY...
		
		//sysProp.setProperty("http.proxyHost", "...");
		//sysProp.setProperty("http.proxyPort", "...");

		// Obtengo el Home de CDSConnector que es la clase de servicio inicial
		try {
			Context jndiContext = new InitialContext();
			Object ref = jndiContext.lookup("EBCConnectorHome");
			EBCConnectorHome home = (EBCConnectorHome) PortableRemoteObject
					.narrow(ref, EBCConnectorHome.class);
			EBCConnector cdsconn = home.create();
			
			// Le indico a la clase de servicio quin soy, si soy usuario, retorna
			// los grupos a los que puedo ingresar, lo cuales vienen en una 
			// estructura de arreglos de string.
			Vector userGroups = cdsconn.getUserGroups("eormeno", "123");
			
			// para verl la lista descoment esto...
//			for (int i=0; i < userGroups.size(); i++) {
//				String group[] = (String[])userGroups.get(i);
//				System.out.println(group[0] + ", " + group[1]);
//			}

			// Aclaracin: existen dos instancias de un componente curso, las cuales
			// estn representadas en la capa de persistencia por dos tablas:
			// '_CURSO' y 'Course'. El primero representa el curso desde un legacy
			// system, es decir, CDS est pensado para ser integrado
			// con un sitema ya existente, en el que "seguramente" existe una tabla
			// que contiene el nombre del curso con algn identificador. Por otro
			// lado est la tabla 'Course' que es la parte persistente de un
			// EJB llamado Course.
			// Por ello, cuando digo 'id de grupo' me estoy refiriendo al ID
			// del primero, y cuando digo 'id de curso' me estoy refiriendo al
			// segundo.

			// Para loguearse hay que especificar en qu grupo quers ingresar
			// para ello tens que indicar el cdigo del curso (del legacy), en particular
			// para "Perl y XML" ste es: tech-01
			cdsconn.login("tech-01");
						
			// Una vez logueado, creo un un manejador de persistencia para EJB.
			// Dado que se accede a travs de mtodos estticos, podr ver en
			// otras partes de mi sistema ste manejador de persistencia. Es decir
			// que lo estoy convirtiendo en un "objeto pblico". De ahora en ms
			// comienzo a manejarme con el manejador de persistencia que, a su vz,
			// encapsula la funcionalidad del conector (cdsconn)
			PersistenceManager env = new EJBPersistence(cdsconn);
			PersistenceSelector.setManager(env);
						
			// Para acceder a cualquier objeto, debo utilizar el mtodo 'getObject(String, int)'
			// en el cual debo indicar el nombre de la clase del objeto y su identificador.
			
			// Para acceder al componente "Course" del curso que estoy interesado,
			// debera hacer: env.getObject("com.dizares.cds.value.Course", id).
			// El problema que tengo es que no s el identificador del objeto
			// Course. Para ello debo "preguntarle" a la clase de servicio "CourseService"
			// cul es el ID del curso donde ste usuario se encuentra logueado.
			
			// Cuando me logue, la clase de servicio almacen el grupo donde
			// estoy (en este caso 'tech-01')
			String grupoActual = env.getGroupID();
			
			// Luego, invoco al mtodo "getCurrentCourse(String)" de la clase
			// de servicio 'CourseService', la cual me retorna un Integer conteniendo el ID del
			// curso que quiero obtener y recibe por parmetro el identificador del grupo
			Integer intID = (Integer)env.callService("CourseService", "getCurrentCourse", new Object[] {grupoActual});
			
			// Recin ahora puedo acceder al objeto Course.
			Course course = (Course)env.getObject("com.dizares.cds.value.Course", intID.intValue());
			
			// Ahora accedo a todo lo que quiero de l. En particular para visualizar
			// todo el rbol XML...
			course.getMain().toOutput(System.out);

		} catch (CommunicationException ce) {
			// Esta excepcin se dispara si no encuentra el servidor. En CDSClient
			// muestra un cuadro de dilogo para ingresar la IP. 
			System.exit(0);
		} catch (NamingException ne) {
			// Esta excepcin se dispara si tiene algn problema con el nombre
			// de la clase de servicio solicitada.
			ne.printStackTrace();
		} catch (CreateException ce) {
			// Esta otra excepcin se dispara si no es posible crear una sesin
			// para el usuario (raro que ocurra).
			System.err.println(ce.getLocalizedMessage());
			System.exit(0);
		} catch (RemoteException re) {
			// Esta otra se produce si hay algn problema con la conexin 
			Exception ex = ((EJBException) re.getCause().getCause())
					.getCausedByException();
			if (ex instanceof SecurityException) {
				System.err.println(ex.getLocalizedMessage());
			}
			System.exit(0);
		}  catch (AclNotFoundException aclNotFound) {
			// Esta otra exepcin se puede disparar cuando invocs al "cdsconn.login"
			// si el usuario que quiere ingresar no tiene un perfil de "docente".
			JOptionPane.showMessageDialog(null, "No system permission",
					"Fatal", JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
	}
}